@@ -12,7 +12,7 @@ from napoleontoolbox.rebalancing import allocation
 
 
 class AbstractRunner(ABC):
-    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, save_model=False, supervision_npy_file_suffix='_supervision.npy', macro_supervision_npy_file_suffix='_macro_supervision.npy', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy', returns_pkl_file_name='returns.pkl', local_root_directory='../data/', user = 'napoleon', n_start = 252):
+    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, save_model=False, supervision_npy_file_suffix='_supervision.npy', macro_supervision_npy_file_suffix='_macro_supervision.npy', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy', returns_pkl_file_name='returns.pkl', local_root_directory='../data/', user = 'napoleon', n_start = 252, transaction_costs = None):
         super().__init__()
         self.supervision_npy_file_suffix=supervision_npy_file_suffix
         self.macro_supervision_npy_file_suffix=macro_supervision_npy_file_suffix
@@ -36,6 +36,7 @@ class AbstractRunner(ABC):
         self.hrp_allocation = True
         self.hrc_allocation = True
         self.mdp_allocation = True
+        self.transaction_costs = None
 
     @abstractmethod
     def runTrial(self,saver, seed,  n, s, low_bound, up_bound):
@@ -76,11 +77,27 @@ class SimpleEnsemblingRunner(AbstractRunner):
         df = df[df.index <= self.running_date]
 
         print('max after filtering ' + str(max(df.index)))
+        ### getting the transaction costs
+        df_res = df.copy()
+        df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
+        if self.transaction_costs is None:
+            print('no transaction costs given')
+            costs_parameters = np.zeros(len(df_ret.columns))
+        else:
+            def get_fees_rate(asset, transaction_costs_dic):
+                try:
+                    fees = transaction_costs_dic[asset]
+                except KeyError as e:
+                    print(e)
+                    return 0.
+                return fees
+            costs_parameters = np.fromiter(map(lambda x: get_fees_rate(x, self.transaction_costs), df_ret.columns),
+                                           dtype=np.float64)        ### getting the transaction costs
 
         check_maxperf_existence = saver.checkRunExistence('mp_' + savingKey)
         if self.maximize_perf and not check_maxperf_existence:
             print('maximizing perf')
-            df_res = df.copy()
+
             portfolio, weight_mat = allocation.rolling_allocation(
                 allocation.maxPerf,
                 df_res,
@@ -91,11 +108,18 @@ class SimpleEnsemblingRunner(AbstractRunner):
                 up_bound=up_bound,
                 filtering_threshold=0.9
             )
-            df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
+
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
-            portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            # # Compute portfolio perf
+            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
+            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
+            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
             saver.saveResults('mp_'+savingKey, portfolio, weight_normalized,last_predicted_weights)
 
+
+
+
         check_drawdown_existence = saver.checkRunExistence('dd_'+savingKey)
         if self.minimize_drawdown and not check_drawdown_existence:
             print('minimizing drawdown')
@@ -112,7 +136,11 @@ class SimpleEnsemblingRunner(AbstractRunner):
             )
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
-            portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            # # Compute portfolio perf
+            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
+            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
+            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
             saver.saveResults('dd_'+savingKey, portfolio, weight_normalized,last_predicted_weights)
 
         check_ew_existence = saver.checkRunExistence('ew_'+savingKey)
@@ -120,7 +148,11 @@ class SimpleEnsemblingRunner(AbstractRunner):
             print('equally weighted')
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
-            portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            # # Compute portfolio perf
+            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
+            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
+            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
             saver.saveResults('ew_'+savingKey, portfolio, weight_normalized,last_predicted_weights)
 
 
@@ -141,7 +173,11 @@ class SimpleEnsemblingRunner(AbstractRunner):
             )
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
-            portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            # # Compute portfolio perf
+            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
+            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
+            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
             saver.saveResults('erc_'+savingKey, portfolio, weight_normalized,last_predicted_weights)
 
 
@@ -162,7 +198,11 @@ class SimpleEnsemblingRunner(AbstractRunner):
             )
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
-            portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            # # Compute portfolio perf
+            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
+            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
+            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
             saver.saveResults('ivp_'+savingKey, portfolio, weight_normalized,last_predicted_weights)
 
 
@@ -183,7 +223,11 @@ class SimpleEnsemblingRunner(AbstractRunner):
             )
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
-            portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            # # Compute portfolio perf
+            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
+            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
+            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
             saver.saveResults('mvp_'+savingKey, portfolio, weight_normalized,last_predicted_weights)
 
 
@@ -206,7 +250,11 @@ class SimpleEnsemblingRunner(AbstractRunner):
             )
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
-            portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            # # Compute portfolio perf
+            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
+            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
+            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
             saver.saveResults('hrp_'+savingKey, portfolio, weight_normalized,last_predicted_weights)
 
 
@@ -224,7 +272,11 @@ class SimpleEnsemblingRunner(AbstractRunner):
             )
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
-            portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            # # Compute portfolio perf
+            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
+            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
+            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
             saver.saveResults('hrc_'+savingKey, portfolio, weight_normalized,last_predicted_weights)
 
         check_mdp_existence = saver.checkRunExistence('mdp_'+savingKey)
@@ -245,5 +297,9 @@ class SimpleEnsemblingRunner(AbstractRunner):
             )
             df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
             weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
-            portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
+            # # Compute portfolio perf
+            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
+            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
+            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
             saver.saveResults('mdp_'+savingKey, portfolio, weight_normalized,last_predicted_weights)
