@@ -12,6 +12,9 @@ import math
 from napoleontoolbox.signal import signal_utility
 
 import numpy as np
+import random
+
+import string
 
 class SignalParalleLauncher():
     def __init__(self,drop_token='', dropbox_backup=True, local_root_directory='../data',user='napoleon',  db_path_suffix = '_run.sqlite', triggers = [True], transaction_costs = [True], normalizations = [False], signal_idiosyncratic_paramaters=[None], table_max_size = 2000):
@@ -24,8 +27,9 @@ class SignalParalleLauncher():
                 for normalization in normalizations:
                     for signal_type, params  in signal_idiosyncratic_paramaters.items():
                         for param in params:
-                            self.args.append((self,self.seed,trigger,signal_type, json.dumps(param, sort_keys=True), transaction_cost, normalization))
-                            self.key_args.append((self.seed, trigger, signal_type, json.dumps(param, sort_keys=True),transaction_cost, normalization))
+                            rand_hash= ''.join(random.choice(string.ascii_lowercase) for i in range(4))
+                            self.args.append((self,rand_hash,trigger,signal_type, json.dumps(param, sort_keys=True), transaction_cost, normalization))
+                            self.key_args.append((rand_hash, trigger, signal_type, json.dumps(param, sort_keys=True),transaction_cost, normalization))
 
         self.key_args.sort()
         self.counter = 1
